#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Hosting;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using EPiServer.Demo.FileManager;
using EPiServer.Demo.FileManager.WebControls;
using EPiServer.Web.Hosting;

namespace EPiServer.Templates.Demo.FileManager
{
    /// <summary>
    /// Displays a list of files in the <see cref="FileManagerControl.CurrentVirtualDirectory"/>
    /// </summary>
    public partial class FileList : ContentUserControlBase
    {
        private bool _isDataBound = false;
        private IComparer<VirtualFileBase> _selectedComparer = new NameComparer(SortDirection.Descending);

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Init"/> event.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void  OnInit(EventArgs e)
        {
         	base.OnInit(e);
            FileManager.CommandExecute += new EventHandler<FileManagerCommandEventArgs>(FileManager_CommandExecute);
        }

        /// <summary>
        /// Sets up a comparer with the previous command. Used to remember sort order when changing folders.
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (!String.IsNullOrEmpty(PreviousCommand))
            {
                SetupSelectedComparer(PreviousCommand);
            }
        }

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.PreRender"/> event.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
            if (!_isDataBound)
            {
                DataBindFileList();
            }
        }

        /// <summary>
        /// Raises a command on the current <see cref="FileManager"/>.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Web.UI.WebControls.CommandEventArgs"/> instance containing the event data.</param>
        protected void SelectFile(object sender, CommandEventArgs e)
        {
            string virtualPath = (string)e.CommandArgument;
            VirtualFileBase file = null;
            // TODO: Check that it's OK to check the /
            if (virtualPath.EndsWith("/"))
            {
                file = HostingEnvironment.VirtualPathProvider.GetDirectory(virtualPath) as VirtualFileBase;
                
            }
            else
            {
                file = HostingEnvironment.VirtualPathProvider.GetFile(virtualPath) as VirtualFileBase;
            }
            int currentIndex = FileManager.SelectedItems.IndexOf(file);
            if (currentIndex >= 0)
            {
                FileManager.SelectedItems.RemoveAt(currentIndex);
            }
            else
            {
                //VirtualFileBase file = HostingEnvironment.VirtualPathProvider.GetFile(virtualPath) as VirtualFileBase;
                FileManager.SelectedItems.Add(file);
            }
            DataBindFileList();
        }

        /// <summary>
        /// Binds the file list to the file repeater control.
        /// </summary>
        protected void DataBindFileList()
        {
            if (FileManager.CurrentVirtualDirectory != null)
            {
                FileRepeater.DataSource = GetFilesAndFolders();
                FileRepeater.DataBind();
                _isDataBound = true;
            }
        }

        /// <summary>
        /// Handles the CommandExecute event of the FileManager control to update the file list when a change occur.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EPiServer.Demo.FileManager.FileManagerCommandEventArgs"/> instance containing the event data.</param>
        private void FileManager_CommandExecute(object sender, FileManagerCommandEventArgs e)
        {
            if (sender == this)
            {
                return;
            }
            if (e.CommandName == FileManagerCommandName.SelectFolder) 
            {
                // Repopulate the file list if the current folder changed.
                DataBindFileList();
            }
        }

        /// <summary>
        /// Gets the file size as a string in human readable format.
        /// </summary>
        /// <param name="fileItem">The file item.</param>
        /// <returns>The file size, or an empty string if the input is not a <see cref="UnifiedFile"/></returns>
        protected string GetFileSizeString(VirtualFileBase fileItem)
        {
            if (fileItem is UnifiedFile)
            {
                return GetFileSizeString(((UnifiedFile)fileItem).Length);
            }
            return String.Empty;
        }

        /// <summary>
        /// Gets the css for a folder or a file extension
        /// </summary>
        /// <param name="fileItem">The file item.</param>
        /// <returns>The folder or a file extension css string or an empty string if the input is not a <see cref="UnifiedFile"/></returns>
        protected string GetFileExtensionCss(object fileItem)
        {
            if (fileItem is UnifiedFile)
            {
                string extension = String.Empty;
                if (!String.IsNullOrEmpty(((UnifiedFile)fileItem).Extension))
                {
                    extension = ((UnifiedFile)fileItem).Extension.Substring(1);
                }
                return String.Format("document {0}Extension", extension);
            }
            if (((VirtualFileBase)fileItem).IsDirectory)
            {
                return "folder";
            }
            return string.Empty;
        }



        /// <summary>
        /// Gets the changed date as a string.
        /// </summary>
        /// <param name="fileItem">The file item.</param>
        /// <returns>The changed date or an empty string if the input is not a <see cref="UnifiedFile"/></returns>
        protected string GetChangedDateString(object fileItem)
        { 
            if (fileItem is UnifiedFile)
            {
                return ((UnifiedFile)fileItem).Changed.ToString("g");
            }
            return string.Empty;
        }

        /// <summary>
        /// Gets the username for the user the file is checked out to.
        /// </summary>
        /// <param name="fileItem">The file item.</param>
        /// <returns>The user name, or an epmty string if the file is not checked out.</returns>
        protected string GetCheckedOutString(VirtualFileBase fileItem)
        {
            IVersioningFile versioningFile = fileItem as IVersioningFile;
            if (versioningFile != null && versioningFile.IsCheckedOut)
            {
                return versioningFile.CheckedOutBy;
            }
            return String.Empty;
        }

        /// <summary>
        /// Gets a sorted collection with files and folders.
        /// </summary>
        /// <returns>A sorted IEnumerable containing VirtualFileBase:s</returns>
        private IEnumerable GetFilesAndFolders()
        {
            List<VirtualFileBase> filesAndDirectories = new List<VirtualFileBase>();
            filesAndDirectories.AddRange(FileManager.CurrentVirtualDirectory.GetDirectories());
            filesAndDirectories.AddRange(FileManager.CurrentVirtualDirectory.GetFiles());

            filesAndDirectories.Sort(SelectedComparer);

            return filesAndDirectories;
        }

        /// <summary>
        /// Determines the sort order when a user clicks on a column heading
        /// </summary>
        protected void LinkButton_Command(Object sender, CommandEventArgs e)
        {
            string commandArgument = (string)e.CommandArgument;

            if (commandArgument.Equals(PreviousCommand))
            {
                Direction = (Direction == SortDirection.Descending) ? SortDirection.Ascending : SortDirection.Descending;
            }
            else
            {
                PreviousCommand = commandArgument;
                Direction = SortDirection.Descending;
            }

            SetupSelectedComparer(commandArgument);
        }

        /// <summary>
        /// Sets up the selected comparer.
        /// </summary>
        /// <param name="commandArgument">The command argument indicating what comparer to set up.</param>
        private void SetupSelectedComparer(string commandArgument)
        {
            switch (commandArgument)
            {
                case "name":
                    SelectedComparer = new NameComparer(Direction);
                    break;
                case "size":
                    SelectedComparer = new SizeComparer(Direction);
                    break;
                case "checkedOut":
                    SelectedComparer = new CheckedOutByComparer(Direction);
                    break;
                case "lastChanged":
                    SelectedComparer = new DateComparer(Direction);
                    break;
            }
        }

        /// <summary>
        /// Gets or sets the previously chosen command.
        /// </summary>
        /// <value>The previously chosen command.</value>
        private string PreviousCommand
        {
            get { return (string)ViewState["PreviousCommand"]; }
            set { ViewState["PreviousCommand"] = value; }
        }

        /// <summary>
        /// Gets or sets the sort direction.
        /// </summary>
        /// <value>The sort direction.</value>
        private SortDirection Direction
        {
            get { return (SortDirection)ViewState["Direction"]; }
            set { ViewState["Direction"] = value; }
        }

        /// <summary>
        /// Gets or sets the currently selected comparer.
        /// </summary>
        /// <value>The currently selected comparer.</value>
        private IComparer<VirtualFileBase> SelectedComparer
        {
            get { return _selectedComparer; }
            set { _selectedComparer = value; }
        }

        #region Comparers

        /// <summary>
        /// Sorts VirtualFileBase:s according to name. Groups files and folders separatly.
        /// </summary>
        class NameComparer : IComparer<VirtualFileBase>
        {
            SortDirection _direction;
            int _compareValue;

            public NameComparer(SortDirection sortDirection)
            {
                _direction = sortDirection;
            }

            public int Compare(VirtualFileBase x, VirtualFileBase y)
            {
                //Check if we are comparing a folder and a file
                if (x.GetType() != y.GetType())
                {
                    _compareValue = (x is VirtualDirectory) ? -1 : 1;
                }
                else
                {
                    _compareValue = x.Name.CompareTo(y.Name);
                }

                if (_direction == SortDirection.Ascending)
                {
                    _compareValue *= -1;
                }

                return _compareValue;
            }
        }

        /// <summary>
        /// Sorts VirtualFileBase:s according to size. Groups files and folders separatly.
        /// </summary>
        class SizeComparer : IComparer<VirtualFileBase>
        {
            SortDirection _direction;
            int _compareValue;

            public SizeComparer(SortDirection sortDirection)
            {
                _direction = sortDirection;
            }

            public int Compare(VirtualFileBase x, VirtualFileBase y)
            {
                //Check if we are comparing a folder and a file
                if (x.GetType() != y.GetType())
                {
                    _compareValue = (x is VirtualDirectory) ? -1 : 1;
                }
                else if (x is UnifiedFile)
                {
                    _compareValue = ((UnifiedFile)x).Length.CompareTo(((UnifiedFile)y).Length);
                }
                else
                {
                    //We are comparing two folders who doesn't have length
                    _compareValue = 0;
                }
                
                if (_direction == SortDirection.Ascending)
                {
                    _compareValue *= -1;
                }

                return _compareValue;
            }
        }

        /// <summary>
        /// Sorts VirtualFileBase:s according to the changed date. Groups files and folders separatly.
        /// </summary>
        class DateComparer : IComparer<VirtualFileBase>
        {
            SortDirection _direction;
            int _compareValue;

            public DateComparer(SortDirection sortDirection)
            {
                _direction = sortDirection;
            }

            public int Compare(VirtualFileBase x, VirtualFileBase y)
            {   
                //Check if we are comparing a folder and a file
                if (x.GetType() != y.GetType())
                {
                    _compareValue = (x is VirtualDirectory) ? -1 : 1;
                }
                else if (x is UnifiedFile)
                {
                    _compareValue = ((UnifiedFile)x).Changed.CompareTo(((UnifiedFile)y).Changed);
                }
                else
                {
                    //We are comparing two folders who doesn't have a last changed property
                    _compareValue = 0;
                }
                
                if (_direction == SortDirection.Ascending)
                {
                    _compareValue *= -1;
                }

                return _compareValue;
            }
        }

        /// <summary>
        /// Sorts VirtualFileBase:s according to who has the file checked out. Groups files and folders separatly.
        /// </summary>
        class CheckedOutByComparer : IComparer<VirtualFileBase>
        {
            SortDirection _direction;
            int _compareValue;

            public CheckedOutByComparer(SortDirection sortDirection)
            {
                _direction = sortDirection;
            }

            public int Compare(VirtualFileBase x, VirtualFileBase y)
            {   
                //Check if we are comparing a folder and a file
                if (x.GetType() != y.GetType())
                {
                    _compareValue = (x is VirtualDirectory) ? -1 : 1;
                }
                else if (x is IVersioningFile)
                {
                    //string xCheckedOut = ((IVersioningFile)x).CheckedOutBy ?? String.Empty;
                    //string yCheckedOut = ((IVersioningFile)y).CheckedOutBy ?? String.Empty;
                    //_compareValue = xCheckedOut.CompareTo(yCheckedOut);

                    _compareValue = String.Compare(((IVersioningFile)x).CheckedOutBy, ((IVersioningFile)y).CheckedOutBy);
                }
                else
                {
                    _compareValue = 0;
                }
             
                if (_direction == SortDirection.Ascending)
                {
                    _compareValue *= -1;
                }

                return _compareValue;
            }
        }

        #endregion
    }
}
