#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Configuration;
using System.Data;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

using EPiServer.Configuration;
using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.DataAccess;
using EPiServer.Filters;

namespace EPiServer.Templates.Demo.Forum.Core
{
    /// <summary>
    /// Help class to create/delete forum items and get information about the forum.
    /// </summary>
    public class Manager
    {
        private static readonly TimeSpan _archiveTimeSpan = new TimeSpan(30, 0, 0, 0); //Thirty days

        /// <summary>
        /// ForumPageTypeName
        /// </summary>
        public static readonly string ForumPageTypeName =               "[Demo] Forum";
        /// <summary>
        /// ThreadPageTypeName
        /// </summary>
        public static readonly string ThreadPageTypeName =              "[Demo] Forum thread";
        /// <summary>
        /// PostPageTypeName
        /// </summary>
        public static readonly string PostPageTypeName =                "[Demo] Forum post";
        /// <summary>
        /// ThreadContainerPageTypeName
        /// </summary>
        public static readonly string ThreadContainerPageTypeName =     "[Demo] Forum thread container";

        /// <summary>
        /// Gets the latest updated threads. Will search through all the forums beneath the supplied root.
        /// </summary>
        /// <param name="pageRef">The root page to look for threads under.</param>
        /// <param name="nrOfThreads">The number of threads to return.</param>
        /// <returns>A PageDataCollection containing threads.</returns>
        public static PageDataCollection GetLatestUpdatedThreads(PageReference pageRef, int nrOfThreads)
        {
            PropertyCriteriaCollection criterias = new PropertyCriteriaCollection();

            criterias.Add("PageTypeName", ThreadContainerPageTypeName, CompareCondition.Equal);

            PageDataCollection pages = DataFactory.Instance.FindPagesWithCriteria(pageRef, criterias);

            PageDataCollection threads = new PageDataCollection();

            foreach (PageData threadContainer in pages)
            {
                foreach (PageData page in DataFactory.Instance.GetChildren(threadContainer.PageLink, LanguageSelector.AutoDetect(), 0, nrOfThreads))
                {
                    threads.Add(page);
                }
            }

            new FilterPropertySort("PageChanged", FilterSortDirection.Descending).Filter(threads);
            new FilterCount(nrOfThreads).Filter(threads);

            return threads;
        }

        /// <summary>
        /// Gets the latest reply in a thread.
        /// </summary>
        /// <param name="threadRef">A reference to the thread.</param>
        /// <returns>The latest reply in the thread.</returns>
        public static PageData GetLatestReply(PageReference threadRef)
        {
            PageDataCollection replies = DataFactory.Instance.GetChildren(threadRef, LanguageSelector.AutoDetect(), 0, 1);
            return replies[0];
        }

        /// <summary>
        /// Creates a reply to a thread.
        /// </summary>
        /// <param name="threadReference">A reference to the thread.</param>
        /// <param name="forumStartPage">A reference to the forum the thread belongs to.</param>
        /// <param name="text">The content of the reply.</param>
        /// <returns>Returns true if the page has been moved from the archive to the active container; Otherwise false.</returns>
        public static bool CreateReply(PageReference threadReference, PageData forumStartPage, string text)
        {
            CreateReplyPage(threadReference, text);
            return UpdateThread(threadReference, forumStartPage);
        }

        /// <summary>
        /// Creates a thread in a forum.
        /// </summary>
        /// <param name="forumStartPage">A reference to the forum</param>
        /// <param name="heading">The heading for the thread.</param>
        /// <param name="text">The content of first entry of the thread.</param>
        /// <returns>The newly created thread page.</returns>
        public static PageData CreateThread(PageData forumStartPage, string heading, string text)
        {
            //Create thread page
            PageData newPage = DataFactory.Instance.GetDefaultPageData((PageReference)forumStartPage["ActiveThreadContainer"], ThreadPageTypeName);
            newPage.PageName = ToSafeString(heading);
            newPage.StartPublish = DateTime.Now.AddMinutes(-1);
            newPage.ArchiveLink = (PageReference)forumStartPage["ArchiveThreadContainer"];
            newPage.StopPublish = DateTime.Now.Add(_archiveTimeSpan);
            newPage["PageChildOrderRule"] = FilterSortOrder.CreatedDescending;
            PageReference pageRef = DataFactory.Instance.Save(newPage, SaveAction.Publish);

            //Create first reply
            CreateReplyPage(pageRef, text);

            return newPage;
        }

        /// <summary>
        /// Creates a new forum.
        /// </summary>
        /// <remarks>
        /// Sets up three subpages to the forum page. One container for active threads, one for sticky threads and one for archived threads.
        /// </remarks>
        /// <param name="parent">The parent to the forum.</param>
        /// <param name="forumName">Name of the forum.</param>
        /// <param name="allowThreads">If set to <c>true</c> the foum allows threads as sub pages.</param>
        /// <param name="allowForums">If set to <c>true</c> the foum allows other forums as sub pages.</param>
        /// <param name="icon">The icon to use for this forum.</param>
        /// <returns>A reference to the newly created forum.</returns>
        public static PageReference CreateForum(PageReference parent, string forumName, bool allowThreads, bool allowForums, string icon)
        {
            PageData newPage = DataFactory.Instance.GetDefaultPageData(parent, ForumPageTypeName);
            newPage.PageName = ToSafeString(forumName);
            newPage.StartPublish = DateTime.Now.AddMinutes(-1);
            newPage["AllowThreads"] = allowThreads;
            newPage["AllowForums"] = allowForums;
            newPage["Icon"] = icon;

            return SetupForumStructure(DataFactory.Instance.Save(newPage, SaveAction.Publish));
        }


        /// <summary>
        /// Sets up the structure for a forum
        /// </summary>
        /// <remarks>
        /// Sets up three subpages to the forum page. One container for active threads, one for sticky threads and one for archived threads.
        /// </remarks>
        /// <param name="forumReference">The reference to the forum.</param>
        public static PageReference SetupForumStructure(PageReference forumReference)
        {
            PageData activeContainer = DataFactory.Instance.GetDefaultPageData(forumReference, ThreadContainerPageTypeName);
            PageData archiveContainer = DataFactory.Instance.GetDefaultPageData(forumReference, ThreadContainerPageTypeName);
            PageData stickyContainer = DataFactory.Instance.GetDefaultPageData(forumReference, ThreadContainerPageTypeName);

            PageData forum = DataFactory.Instance.GetPage(forumReference).CreateWritableClone();

            activeContainer.PageName = "Active";
            archiveContainer.PageName = "Archive";
            stickyContainer.PageName = "Sticky";
            activeContainer.VisibleInMenu = false;
            archiveContainer.VisibleInMenu = false;
            stickyContainer.VisibleInMenu = false;
            activeContainer["PageChildOrderRule"] = FilterSortOrder.ChangedDescending;
            archiveContainer["PageChildOrderRule"] = FilterSortOrder.ChangedDescending;
            stickyContainer["PageChildOrderRule"] = FilterSortOrder.ChangedDescending;

            forum["ActiveThreadContainer"] = DataFactory.Instance.Save(activeContainer, SaveAction.Publish);
            forum["ArchiveThreadContainer"] = DataFactory.Instance.Save(archiveContainer, SaveAction.Publish);
            forum["StickyThreadContainer"] = DataFactory.Instance.Save(stickyContainer, SaveAction.Publish);
            DynamicProperty forumStart = DynamicProperty.Load(forum.PageLink, "ForumStart");
            forumStart.PropertyValue.Value = forum.PageLink;
            forumStart.Save();

            return DataFactory.Instance.Save(forum, SaveAction.Publish);
        }


        /// <summary>
        /// Deletes a reply.
        /// </summary>
        /// <param name="replyReference">A reference to the reply.</param>
        /// <param name="threadReference">A reference to the page.</param>
        public static void DeleteReply(PageReference replyReference, PageReference threadReference)
        {
            PageData thread = DataFactory.Instance.GetPage(threadReference).CreateWritableClone();
            thread["NumberOfReplies"] = (int)thread["NumberOfReplies"] - 1;
            DataFactory.Instance.Save(thread, SaveAction.Publish);

            DeleteItem(replyReference);
        }

        /// <summary>
        /// Deletes a thread.
        /// </summary>
        /// <param name="threadReference">A reference to the thread.</param>
        public static void DeleteThread(PageReference threadReference)
        {
            DeleteItem(threadReference);
        }

        /// <summary>
        /// Deletes a forum.
        /// </summary>
        /// <param name="forumReference">A reference to the forum.</param>
        public static void DeleteForum(PageReference forumReference)
        {
            DeleteItem(forumReference);
        }

        /// <summary>
        /// Deletes a page.
        /// </summary>
        /// <param name="pageReference">A reference to the page.</param>
        private static void DeleteItem(PageReference pageReference)
        {
            DataFactory.Instance.Delete(pageReference, true);
        }

        /// <summary>
        /// Creates the actual page that contains the reply data.
        /// </summary>
        /// <param name="threadReference">A reference to the thread that this is a reply to.</param>
        /// <param name="text">The content of the reply.</param>
        private static void CreateReplyPage(PageReference threadReference, string text)
        {
            PageData newPage = DataFactory.Instance.GetDefaultPageData(threadReference, PostPageTypeName, EPiServer.Security.AccessLevel.NoAccess);
            newPage.PageName = DateTime.Now.AddMinutes(-1).ToString("yyyyMMdd'T'HHmmss");
            newPage["MainBody"] = ToSafeString(text.Trim().Replace(System.Environment.NewLine, "<br/>"));
            newPage.StartPublish = DateTime.Now.AddMinutes(-1);
            DataFactory.Instance.Save(newPage, EPiServer.DataAccess.SaveAction.Publish, EPiServer.Security.AccessLevel.NoAccess);
        }

        /// <summary>
        /// Updates the changed date and number of replies on a thread when a reply is made.
        /// </summary>
        /// <param name="threadReference">A reference to the thread.</param>
        /// <param name="forumStartPage">A reference to the forum the thread belongs to.</param>
        /// <returns>Returns true if the page has been moved from the archive to the active container; Otherwise false.</returns>
        private static bool UpdateThread(PageReference threadReference, PageData forumStartPage)
        {
            PageData thread = DataFactory.Instance.GetPage(threadReference).CreateWritableClone();
            thread["PageChangedOnPublish"] = true;
            thread.StopPublish = DateTime.Now.Add(_archiveTimeSpan);
            if (thread["NumberOfReplies"] == null)
            {
                thread["NumberOfReplies"] = 1;
            }
            else
            {
                thread["NumberOfReplies"] = (int)thread["NumberOfReplies"] + 1;
            }

            DataFactory.Instance.Save(thread, SaveAction.Publish, EPiServer.Security.AccessLevel.NoAccess);

            if (thread.ParentLink.CompareToIgnoreWorkID((PageReference)forumStartPage["ArchiveThreadContainer"]))
            {
                DataFactory.Instance.Move(thread.PageLink, (PageReference)forumStartPage["ActiveThreadContainer"]);
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reomves non allowed tags from the string
        /// </summary>
        /// <param name="input">A string containing html-tags.</param>
        /// <returns>The input washed from non allowed html-tags.</returns>
        private static string ToSafeString(string input)
        {
            string[] safeTags = null;
            string safeString = input.Replace("<", "&lt;").Replace(">", "&gt;");

            if (!String.IsNullOrEmpty(Settings.Instance.UISafeHtmlTags))
            {
                safeTags = Settings.Instance.UISafeHtmlTags.Split(',');
                if (safeTags == null || safeTags.Length == 0 || (safeTags.Length == 1 && safeTags[0].Length == 0))
                {
                    return safeString;
                }
            }
            return Regex.Replace(safeString, @"&lt;(/?(" + String.Join("|", safeTags) + @")/?\s*)&gt;", "<$1>", RegexOptions.IgnoreCase);
        }
    }
}
