#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

#region Known Limitations
/*
The template has the following known limitations:

- Usability limitations
  CalendarView selection won't work without javascript.
  
- Performance Limitations
  Using two calendar lists may not be optimal performance wise.

- Architecture limitations
  Calendar item pages has to be visible in menu. 
*/
#endregion

using System;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.DataAccess;
using EPiServer.Web.WebControls;

namespace EPiServer.Templates.Demo.Pages
{
    public partial class Calendar : TemplatePage
    {
        private static readonly string dateFormat = "dd MMM yyyy";
        private static readonly string dateFormatTime = "dd MMM yyyy HH:mm";
        private static readonly string timeFormat = "HH:mm";

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            CalendarView.SelectionChanged += new EventHandler(CalendarView_SelectionChanged);
            CalendarView.DayRender += new DayRenderEventHandler(CalendarView_DayRender);
            CalendarView.VisibleMonthChanged += new MonthChangedEventHandler(CalendarView_VisibleMonthChanged);

            if (!IsPostBack)
            {
                // Default begin date is todays date.
                CalendarList.BeginDate = DateTime.Now;
                CalendarList.DataBind();

                // MonthlyItems is a hidden calendar list used to calculate what days to highlight in CalenderView.
                // We use the first of current month as begin date but substract 6 days since CalendarView can show up to 6 days of previous month.
                MonthlyItems.BeginDate = new DateTime(CalendarView.TodaysDate.Year, CalendarView.TodaysDate.Month, 1).AddDays(-6);
                // NumberOfDaysToRender is calculated to 6 + 31 + 13 = 50.
                // 6 = CalendarView can show up to 6 days of previous month.
                // 31 = Maximum number of days in a month.
                // 13 = CalendarView can show up to 13 days of next month.
                MonthlyItems.NumberOfDaysToRender = 50;
                MonthlyItems.DataBind();
            }
        }

        /// <summary>
        /// Setups the data for the Calendar list.
        /// </summary>
        protected void SetupData()
        {
            CalendarList.SelectedDates = CalendarView.SelectedDates;
            CalendarList.DataBind();
        }

        /// <summary>
        /// Switch to display event information when an item in the calendar list is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Web.UI.WebControls.CommandEventArgs"/> instance containing the event data.</param>
        protected void LinkButton_Command(Object sender, CommandEventArgs e)
        {
            PageReference pageRef = new PageReference((string)e.CommandArgument);
            EventMainBody.PageLink = pageRef;
            EventPageName.PageLink = pageRef;
            
            EventDate.Text = GetEventDateString(GetPage(pageRef));
            
            CalendarView.SelectedDates.Clear();

            SetVisiblePanel(false);
        }

        #region CalendarView Event Handlers
        
        private void CalendarView_SelectionChanged(object sender, EventArgs e)
        {
            SetupData();
            SetVisiblePanel(true);
        }

        private void CalendarView_VisibleMonthChanged(object sender, MonthChangedEventArgs e)
        {
            // Set BeginDate for hidden calendar list.
            // Substract 6 days since calendar view can show up to 6 days of previous month.
            MonthlyItems.BeginDate = e.NewDate.AddDays(-6);
        }

        private void CalendarView_DayRender(object sender, DayRenderEventArgs e)
        {
            TableCell cell = e.Cell;
            
            if (MonthlyItems.DateIsActive(e.Day.Date))
            {
                cell.BackColor = System.Drawing.Color.Empty;
                cell.ForeColor = System.Drawing.Color.White;
                cell.CssClass += " highlighted";
            }
        }
        
        #endregion

        #region Helper methods

        /// <summary>
        /// Gets a display string for the selected period.
        /// </summary>
        /// <returns>A string with start and end date of selected period</returns>
        protected string GetSelectedPeriod()
        {
            StringBuilder returnValue = new StringBuilder();

            if (CalendarList.SelectedDates.Count == 1)
            {
                returnValue.Append(CalendarList.SelectedDates[0].ToString(dateFormat));
            }
            else if (CalendarList.SelectedDates.Count > 1)
            {
                returnValue.Append(CalendarList.SelectedDates[0].ToString(dateFormat));
                returnValue.Append(" &#8722; ");
                returnValue.Append(CalendarList.SelectedDates[CalendarList.SelectedDates.Count - 1].ToString(dateFormat));
            }
            else
            {
                DateTime beginDate = CalendarList.BeginDate;
                returnValue.Append(beginDate.ToString(dateFormat));
                returnValue.Append(" &#8722; ");
                returnValue.Append(beginDate.AddDays(CalendarList.NumberOfDaysToRender).ToString(dateFormat));
            }

            return returnValue.ToString();
        }
        
        /// <summary>
        /// Gets a display string containing the start and stop date of this event.
        /// </summary>
        /// <returns>A string containing the start and stop date of this event.</returns>
        private string GetEventDateString(PageData calendarEvent)
        {
            StringBuilder returnValue = new StringBuilder();

            DateTime startDate = (DateTime)calendarEvent["EventStartDate"];
            DateTime stopDate = (DateTime)calendarEvent["EventStopDate"];

            if (startDate.Date == stopDate.Date)
            {
                returnValue.Append(startDate.ToString(dateFormatTime));
                returnValue.Append(" &#8722; ");
                returnValue.Append(stopDate.ToString(timeFormat));
            }
            else if (((int)calendarEvent["ValidDays"]) != 0)
            {
                returnValue.Append(startDate.ToString(timeFormat));
                returnValue.Append(" &#8722; ");
                returnValue.Append(stopDate.ToString(timeFormat));
            }
            else
            {
                returnValue.Append(startDate.ToString(dateFormatTime));
                returnValue.Append(" &#8722; ");
                returnValue.Append(stopDate.ToString(dateFormatTime));
            }

            return returnValue.ToString();
        }


        private void SetVisiblePanel(bool showPeriodPanel)
        {
            DisplayPeriod.Visible = showPeriodPanel;
            DisplayEvent.Visible = !showPeriodPanel;
        }

        /// <summary>
        /// Creates the text used to display a specific day of an event. Returns a string containing the time for the calendar event.
        /// Sets up tooltip when needed and sets the relevant css class for the label EventDate.
        /// </summary>
        /// <param name="calendarEvent">A CalendarEventTemplateContainer for this specific event and day.</param>
        /// <returns>A string containing the time for the calendar event.</returns>
        protected string GetTextAndUpdateLabelProperties(CalendarEventTemplateContainer calendarEvent)
        {
            Label eventDate = (Label)calendarEvent.FindControl("EventDate");

            StringBuilder returnValue = new StringBuilder();
            StringBuilder tooltip = new StringBuilder();
            
            tooltip.Append(DateTime.Parse(calendarEvent.StartDate + " " + calendarEvent.StartTime).ToString(dateFormatTime));
            tooltip.Append(" - ");
            tooltip.Append(DateTime.Parse(calendarEvent.StopDate + " " + calendarEvent.StopTime).ToString(dateFormatTime));

            eventDate.ToolTip = tooltip.ToString();
            
            //Depending on what kind of event it is (single day, multiple day or recurring), we want to display the time differently.
            // The event is a single day event, or a recurring event.
            if (calendarEvent.StartDate == calendarEvent.StopDate || ((int)calendarEvent.CurrentPage["ValidDays"]) != 0)
            {
                eventDate.CssClass = "eventDate";
                eventDate.ToolTip = String.Empty;

                returnValue.Append(calendarEvent.StartTime.Substring(0, 5));
                returnValue.Append(" &#8722; ");
                returnValue.Append(calendarEvent.StopTime.Substring(0, 5));
            }
            // The event is a multi day event, and we're on the first day.
            else if (calendarEvent.CurrentDate.Date == DateTime.Parse(calendarEvent.StartDate))
            {
                eventDate.CssClass = "eventDateIconRight";
                returnValue.Append(calendarEvent.StartTime.Substring(0, 5));
                
            }
            // The event is a multi day event, and we're on the last day.
            else if (calendarEvent.CurrentDate.Date == DateTime.Parse(calendarEvent.StopDate))
            {
                eventDate.CssClass = "eventDateIconLeft";
                returnValue.Append(calendarEvent.StopTime.Substring(0, 5));
            }
            // The event is a multi day event, and we're neither on the first or the last day.
            else
            {
                eventDate.CssClass = "eventDateIcon";
            }
            return returnValue.ToString();
        }

        #endregion
    }
}

