#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Web;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

using EPiServer;
using EPiServer.Core;
using EPiServer.Filters;
using EPiServer.Web.WebControls;

namespace EPiServer.Templates.Demo.Units.Placeable
{
    /// <summary>
    /// Presents a listing of pages in an alphanumeric order. 
    /// </summary>
	public abstract partial class AlphanumericListing : UserControlBase
	{
        // "PageAlphanumericChars" (PropertyString)[Optional, Default value="ABCDEFGHIJKLMNOPQRSTUVWXYZ"]
        //  Used to define a custom character set.
        private static readonly string _alphanumericCharsPropertyName = "PageAlphanumericChars";
        
        // "IndexContainer" (PropertyPageReference)[Optional, Default value=Start page for site] defines the root page for pagelist.
        private static readonly string _indexContainerPropertyName = "IndexContainer";
        
        private ArrayList _alphanumericRanges;
        private string _alphanumerics;

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"></see> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"></see> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            AlphabeticalLinks.DataBind();
            if (!IsPostBack)
            {
                SearchDataSource.DataBind();
            }
        }
        
        /// <summary>
        /// Click event handler for the alphabetical list to sort pages by.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void ChangeLetters(object sender, System.EventArgs e)
        {
            LinkButton link = (LinkButton)sender;
            foreach (char letter in SplitAlphanumericRange(link.Text))
            {
                AddLetter(letter);
            }

            AlphabeticalList.Visible = true;
            AlphabeticalList.DataBind();
        }

        /// <summary>
        /// Gets the list of all characters used.
        /// </summary>
        public ArrayList AlphanumericRanges
        {
            get
            {
                if (_alphanumericRanges == null)
                {
                    _alphanumericRanges = CreateAlphanumericRanges(Alphanumerics);
                }
                return _alphanumericRanges;
            }
        }

        /// <summary>
        /// The root for the search.
        /// </summary>
        public PageReference StartingPoint
        {
            get
            {
                object indexContainer = CurrentPage[_indexContainerPropertyName];
                
                if (indexContainer == null)
                {
                    return PageReference.StartPage;
                }
                else
                {
                    return PageReference.Parse(indexContainer.ToString());
                }
            }
        }

        /// <summary>
        /// Checks if page has a definition for the "PageAlphanumericChars" property.
        /// If not uses standard alphabet for default languages: EN, SV, FI, NO, DK.
        /// </summary>
        /// <returns>A string with the text used to create alphabetical linkbuttons.</returns>
        protected string Alphanumerics
        {
            get
            {
                if (!String.IsNullOrEmpty(_alphanumerics)) 
                {
                    return _alphanumerics;
                }

                _alphanumerics = (string)CurrentPage[_alphanumericCharsPropertyName];

                if (_alphanumerics == null)
                {
                    switch (CurrentPage.LanguageID)
                    {
                        case "sv":
                        case "fi":
                            _alphanumerics = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
                            break;
                        case "no":
                        case "dk":
                            _alphanumerics = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
                            break;
                        default:	// use English alphabet by default
                            _alphanumerics = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
                            break;
                    }
                }
                return _alphanumerics;
            }
        }

        /// <summary>
        /// Splits an alphabetical list into parts of 5.
        /// </summary>
        /// <param name="chars">The list of characters used.</param>
        /// <returns>An arraylist with the text used to create alphabetical linkbuttons.</returns>
		protected ArrayList CreateAlphanumericRanges(string chars)
		{
			const int rangeStep = 5;
			const int minCharsInRange = 3;

			int maxPos = chars.Length;
			int rangeEndPos;
			
            ArrayList ranges = new ArrayList();

			for(int pos = 0; pos < maxPos; pos = rangeEndPos + 1)
			{
				rangeEndPos = pos + rangeStep - 1;
                if (rangeEndPos >= maxPos || (maxPos - rangeEndPos < minCharsInRange))
                {
                    rangeEndPos = maxPos - 1;
                }
				ranges.Add(chars[pos].ToString() + " - " + chars[rangeEndPos].ToString());
			}
            
            return ranges;
		}

        /// <summary>
        /// Creates a character list from the alphanumeric range.
        /// </summary>
        /// <param name="range">A range of characters. Example: A - E</param>
        /// <returns>An array of characters containing all the characters in the range. Example: ABCDE</returns>
		protected char[] SplitAlphanumericRange(string range)
		{
            int start = Alphanumerics.IndexOf(range[0]);
            int stop = Alphanumerics.IndexOf(range[range.Length - 1]);

            return Alphanumerics.Substring(start, stop - start + 1).ToCharArray();
		}

        /// <summary>
        /// Adds a criteria stating what character the page name should start with.
        /// </summary>
        /// <param name="letter">The character the page name should start with.</param>
		protected void AddLetter(char letter)
		{
            PropertyCriteriaControl criteria = new PropertyCriteriaControl();
        
		    criteria.Condition = CompareCondition.StartsWith;
			criteria.Type = PropertyDataType.String;
			criteria.Value = letter.ToString().ToUpper();
			criteria.Name = "PageName";
        
            SearchDataSource.LanguageBranches = CurrentPage.LanguageBranch;
            SearchDataSource.Criteria.Add(criteria);
        }
    }
}
