#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using EPiServer.Core;
using EPiServer.Web.Hosting;
using EPiServer.Templates.Demo.Units.Static;
using EPiServer.Web.WebControls;
using EPiServer.Filters;

namespace EPiServer.Templates.Demo.Workroom.Core
{
    /// <summary>
    /// Base class for pages with workroom functionality
    /// </summary>
    public class WorkroomPageBase : TemplatePage
    {
        private PageReference _workroomStartPageLink = null;
        private PageData _workroomStartPage = null;

        private string _fileFolderRoot;
        private bool? _editableContent;
        private bool _viewStateLoaded = false;

        //Set to false to have the workroom startpage as root of the submenu
        private readonly bool _showWorkroomListPage = false;
        private readonly string _workroomListPageTypeName = "[Demo] Workroom list";

        /// <summary>
        /// Gets a value indicating if the ViewState has been restored.
        /// </summary>
        /// <value><c>true</c> if view state is loaded; otherwise, <c>false</c>.</value>
        public bool ViewStateLoaded
        {
            get { return _viewStateLoaded; }
        }

        /// <summary>
        /// Restores view-state information from a previous page request that was saved by the <see cref="M:System.Web.UI.Control.SaveViewState"/> method.
        /// </summary>
        /// <param name="savedState">An <see cref="T:System.Object"/> that represents the control state to be restored.</param>
        protected override void LoadViewState(object savedState)
        {
            base.LoadViewState(savedState);
            _viewStateLoaded = true;
        }

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            //Loacate the submenu PageTree-control.
            PageTree pagetree = Master.FindControl("MainRegion$SubMenu$Menu") as PageTree;

            //Setup the submenu according to preference.
            if (pagetree != null && CurrentPage.PageTypeName != _workroomListPageTypeName)
            {
                if (_showWorkroomListPage)
                {
                    pagetree.PageLink = WorkroomStartPage.ParentLink;
                    pagetree.NumberOfLevels = 3;
                    pagetree.EnableVisibleInMenu = false;
                    pagetree.Filter += new FilterEventHandler(pagetree_Filter);
                }
                else
                {
                    //If the StratPage is in edtiable mode the dynamic properties won't exist
                    //therefore we do this workaround.
                    if (CurrentPage["WorkroomStart"] != null)
                    {
                        pagetree.PageLink = WorkroomStartPageLink;
                    }
                    else
                    {
                        pagetree.PageLink = CurrentPage.PageLink;
                    }
                    pagetree.NumberOfLevels = 2;
                }
                pagetree.DataBind();
            }
        }

        private void pagetree_Filter(object sender, EPiServer.Filters.FilterEventArgs e)
        {
            PageData page;
            for (int i = e.Pages.Count; i > 0; i--) 
            {
                page = e.Pages[i-1];
                if (!page.PageLink.CompareToIgnoreWorkID(WorkroomStartPageLink) && !page.ParentLink.CompareToIgnoreWorkID(WorkroomStartPageLink) || (page.ParentLink.CompareToIgnoreWorkID(WorkroomStartPageLink) && !page.VisibleInMenu)) 
                {
                    e.Pages.Remove(page);
                }
            }
        }

        /// <summary>
        /// Gets or sets the work room file folder root.
        /// </summary>
        /// <value>Path to root folder for files.</value>
        public string FileFolderRoot
        {
            get
            {
                if (String.IsNullOrEmpty(_fileFolderRoot))
                {
                    _fileFolderRoot = VirtualPathUtility.AppendTrailingSlash(VirtualPathHandler.PageDirectoryRootVirtualPath);
                    _fileFolderRoot = VirtualPathUtility.AppendTrailingSlash(VirtualPathUtility.Combine(_fileFolderRoot, WorkroomStartPage["PageFolderID"].ToString()));
                }
                return _fileFolderRoot;
            }
        }


        /// <summary>
        /// Gets or sets the writeable state of the CurrentPage property.
        /// If the to true the <see cref="EPiServer.Core.PageData"/> returned from <see cref="CurrentPage"/> will be writable.
        /// </summary>
        /// <remarks>
        /// The value is stored in ViewState and persisted between requests.
        /// Setting the value to false does <c>not</c> make the <see cref="EPiServer.Core.PageData"/> returned from <see cref="CurrentPage"/> read-only. 
        /// </remarks>
        public bool EditableContent
        {
            get
            {
                if (!_editableContent.HasValue && ViewStateLoaded)
                {
                    _editableContent = (bool)(ViewState["EditableContent"] ?? false);
                }
                return _editableContent.GetValueOrDefault(false);
            }
            set 
            { 
                ViewState["EditableContent"] = _editableContent = value;
            }
        }

        /// <summary>
        /// Gets the <see cref="EPiServer.Core.PageData"/> for the current page.
        /// If <see cref="EditableContent"/> is <c>true</c> the <see cref="EPiServer.Core.PageData"/> returned will be writable.
        /// </summary>
        public override PageData CurrentPage
        {
            get
            {
                if (EditableContent && base.CurrentPage.IsReadOnly)
                {
                    base.CurrentPage = base.CurrentPage.CreateWritableClone();
                }
                return base.CurrentPage;
            }
            set
            {
                base.CurrentPage = value;
            }
        }

        /// <summary>
        /// Gets the <see cref="EPiServer.Core.PageReference"/> for the start page of this workroom.
        /// </summary>
        /// <remarks>An exception is thrown if the WorkroomStart property is not defined.</remarks>
        public virtual PageReference WorkroomStartPageLink
        {
            get 
            {
                if (PageReference.IsNullOrEmpty(_workroomStartPageLink))
                {
                    _workroomStartPageLink = CurrentPage["WorkroomStart"] as PageReference;
                    if (PageReference.IsNullOrEmpty(_workroomStartPageLink))
                    {
                        throw new EPiServerException("The property \"WorkroomStart\" must be defined for workroom pages");
                    }
                }
                return _workroomStartPageLink;

            }
        }

        /// <summary>
        /// Gets the <see cref="EPiServer.Core.PageData"/> for the start page of this workroom.
        /// </summary>
        /// <remarks>An exception is thrown if the WorkroomStart property is not defined.</remarks>
        public virtual PageData WorkroomStartPage
        {
            get 
            {
                if (_workroomStartPage == null)
                {
                    _workroomStartPage = GetPage(WorkroomStartPageLink);
                }
                return _workroomStartPage;
            }
        }

        #region Workroom security

        /// <summary>
        /// Gets a value indicating whether the current user has at least the supplied membership level for the Workroom.
        /// </summary>
        /// <param name="membershipLevel">The membership level to test.</param>
        /// <returns><c>true</c> if the user has sufficent membership level; otherwise <c>false</c>.</returns>
        public bool QueryDistinctMembershipLevel(MembershipLevel membershipLevel) 
        {
            return Membership.QueryDistinctMembershipLevel(CurrentPage, membershipLevel);
        }

        /// <summary>
        /// The <see cref="EPiServer.Templates.Demo.Workroom.Core.MembershipLevel"/> required to access this page.
        /// </summary>
        public virtual MembershipLevel RequiredMembershipLevel() 
        {
            return MembershipLevel.Read;
        }

        /// <summary>
        /// The <see cref="EPiServer.Templates.Demo.Workroom.Core.MembershipLevel"/> required to access this page.
        /// </summary>
        public override EPiServer.Security.AccessLevel RequiredAccess()
        {
            return Membership.ConvertAccessLevel(RequiredMembershipLevel());
        }

        #endregion

    }
}
