#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

#region Known Limitations
/*
The template has no known limitations.
*/
#endregion

using System;

using EPiServer.Core;
using EPiServer.DataAccess;
using EPiServer.Templates.Demo.Workroom.Core;
using EPiServer.Web.WebControls;

namespace EPiServer.Templates.Demo.Workroom.Pages
{   
    /// <summary>
    /// Shows the current selected news item.
    /// List all news items.
    /// </summary>
    public partial class NewsListPage : WorkroomPageBase
    {
        private enum DisplayMode { View, Add, Edit };
        private static readonly string _newsItemPageTypeName = "[Public] News item";
        private PageData _selectedNewsItem;
        private DisplayMode _currentDisplayMode = DisplayMode.View;

        /// <summary>
        /// Hides the edit and delete buttons if the current user hasn't got at least Modify membership.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            if (!IsPostBack)
            {
                DeleteButton.OnClientClick = string.Format("if(!confirm('{0}'))return false;", Translate("/common/messages/confirmdelete")); 
                NewsListUnit.DataBind();
            }

            UpdateDisplayTexts(NewsListUnit.SelectedPageID);
            SetDisplayMode();

        }

        /// <summary>
        /// Updates the PageLink for MainBody and MainIntro.
        /// </summary>
        /// <param name="selectedNewsPageRef">A PageReference to the page which we want to list data from.</param>
        public void UpdateDisplayTexts(PageReference selectedNewsPageRef)
        {
            PageNameProp.PageLink = MainBodyProp.PageLink = selectedNewsPageRef;
            PageNameProp.DataBind();
            MainBodyProp.DataBind();
        }

        /// <summary>
        /// Toggles the visiblility of the create and display panels and buttons.
        /// </summary>
        private void SetDisplayMode()
        {
            switch (CurrentDisplayMode)
            {
                case DisplayMode.Add:
                    ViewPanel.Visible = false;
                    EditPanel.Visible = true;
                    NewsListUnit.AddButtonVisible = false;
                    EditPanelCaption.InnerHtml = Translate("/workroom/newslistpage/createheading");
                    EditPageName.Text = String.Empty;
                    EditMainBody.Text = String.Empty;
                    break;

                case DisplayMode.Edit:
                    ViewPanel.Visible = false;
                    EditPanel.Visible = true;
                    NewsListUnit.AddButtonVisible = false;
                    EditPanelCaption.InnerHtml = Translate("/workroom/newslistpage/editheading");
                    break;

                default:
                    ViewPanel.Visible = true;
                    EditPanel.Visible = false;

                    //No news items available
                    if (CurrentPage.PageLink.CompareToIgnoreWorkID(NewsListUnit.SelectedPageID))
                    {
                        DeleteButton.Visible = false;
                        EditButton.Visible = false;
                    }
                    else
                    {
                        DeleteButton.Visible = true;
                        EditButton.Visible = true;
                    }
                    //Check rights
                    if (Membership.QueryDistinctMembershipLevel(GetSelectedNewsItem(NewsListUnit.SelectedPageID), MembershipLevel.Modify))
                    {
                        NewsListUnit.AddButtonVisible = true;
                    }
                    else
                    {
                        DeleteButton.Visible = false;
                        EditButton.Visible = false;
                        NewsListUnit.AddButtonVisible = false;
                    }
                    
                    break;
            }
        }

        #region Button click event handlers

        /// <summary>
        /// Handles the Click event of the CreateNewsItem button.
        /// Creates a page of the type "[Public] News item" and places it below the current page.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Save_Click(object sender, EventArgs e)
        {
            if (IsValid)
            {
                switch (CurrentDisplayMode)
                {
                    case DisplayMode.Add:
                        PageData newPage = DataFactory.Instance.GetDefaultPageData(CurrentPageLink, _newsItemPageTypeName);
                        newPage.PageName = EditPageName.Text;
                        newPage["MainBody"] = EditMainBody.Text;
                        newPage.StartPublish = DateTime.Now.AddMinutes(-1);
                        NewsListUnit.SelectedPageID = DataFactory.Instance.Save(newPage, SaveAction.Publish);

                        break;

                    case DisplayMode.Edit:
                        PageData page = GetSelectedNewsItem(NewsListUnit.SelectedPageID).CreateWritableClone();

                        try
                        {
                            page["PageName"] = EditPageName.Text;
                            page["MainBody"] = EditMainBody.Text;

                            DataFactory.Instance.Save(page, SaveAction.Publish);
                        }
                        catch (Exception ex)
                        {
                            Page.Validators.Add(new StaticValidator(ex.Message));
                            return;
                        }

                        break;

                    default:
                        break;
                }
                CurrentDisplayMode = DisplayMode.View;
                NewsListUnit.DataBind();
            }       
        }

        /// <summary>
        /// Handles the Click event of the DeleteNewsItem button.
        /// Deletes the page for the selected news item.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Delete_Click(object sender, EventArgs e)
        {
            DataFactory.Instance.Delete(MainBodyProp.PageLink, true);

            NewsListUnit.SelectedPageID = null;
            //Redirect to self to reload everything after a delete.
            Response.Redirect(CurrentPage.LinkURL, true);
        }

        /// <summary>
        /// Handles the Click event of the Add button of the news-list.
        /// View the add/edit form.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Add_Click(object sender, EventArgs e)
        {
            CurrentDisplayMode = DisplayMode.Add;
        }

        /// <summary>
        /// Handles the Click event of the EditNewsItem button.
        /// Loads the proper values to the editable boxes, and toggles visiblility between the edit panel and the display panel.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Edit_Click(object sender, EventArgs e)
        {
            EditPageName.Text = (string)PageNameProp.InnerProperty.Value;
            EditMainBody.Text = (string)MainBodyProp.InnerProperty.Value;

            CurrentDisplayMode = DisplayMode.Edit;
        }

        /// <summary>
        /// Handles the Click event of the CancelEdit Button.
        /// Toggles visiblility between the edit panel and the display panel.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Cancel_Click(object sender, EventArgs e)
        {
            CurrentDisplayMode = DisplayMode.View;
        }

        #endregion

        #region Private helper methods

        /// <summary>
        /// Gets the selected news item.
        /// </summary>
        private PageData GetSelectedNewsItem(PageReference selectedPageRef)
        {
            if (_selectedNewsItem == null)
            {
                _selectedNewsItem = DataFactory.Instance.GetPage(selectedPageRef);
            }
            return _selectedNewsItem;
        }

        /// <summary>
        /// Set what parts of the page to show.
        /// </summary>
        private DisplayMode CurrentDisplayMode
        {
            get
            {
                if (ViewState["CurrentDisplayMode"] != null)
                {
                    _currentDisplayMode = (DisplayMode)ViewState["CurrentDisplayMode"];
                }
                return _currentDisplayMode;
            }
            set { ViewState["CurrentDisplayMode"] = _currentDisplayMode = value; }
        }

        #endregion
    }
}
