#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

#region Known Limitations
/*
The template has no known limitations.
*/
#endregion

using System;
using System.Web;
using System.Web.Hosting;
using System.Web.UI.WebControls;

using EPiServer.Core;
using EPiServer.DataAccess;
using EPiServer.Filters;
using EPiServer.Security;
using EPiServer.Templates.Demo.Workroom.Core;
using EPiServer.Web;
using EPiServer.Web.Hosting;

namespace EPiServer.Templates.Demo.Workroom.Pages
{
    /// <summary>
    /// List all avaliable work rooms.
    /// Used to create new workrooms.
    /// </summary>
    public partial class WorkroomList : WorkroomPageBase
    {
        private PageDataCollection _workrooms;

        /// <summary>
        /// Lists all avaliable workroom templates from the workroom template container.
        /// Displayes a description for the selected workroom template.
        /// Disables the create workroom button if the user lacks create rights.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"></see> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            if (!IsPostBack)
            {
                WorkroomListPanel.Visible = true;
                NewWorkroomPanel.Visible = false;

                PageDataCollection templatePages = DataFactory.Instance.GetChildren((PageReference)CurrentPage["TemplateContainer"]);

                foreach (PageData template in templatePages)
                {
                    SelectTemplateDropDown.Items.Add(new ListItem(template.PageName, template.PageLink.ID.ToString()));
                }
            }
            else if (!String.IsNullOrEmpty(SelectTemplateDropDown.SelectedValue))
            {
                PageData page = DataFactory.Instance.GetPage(PageReference.Parse(SelectTemplateDropDown.SelectedValue));
                DescriptionLabel.Text = (string)page["MainBody"];
            }

            if (CurrentPage.ACL.QueryDistinctAccess(AccessLevel.Publish))
            {
                ShowCreatePanelButton.Visible = true;
            }

            WorkroomPageList.DataBind();
        }

        /// <summary>
        /// Copies the selected workroom template to create an active workrooms. 
        /// Sets up the name of the new workroom.
        /// </summary>
        /// <param name="templateRef">A reference to the selected workroom template</param>
        /// <returns>The new workroom startpage.</returns>
        private PageData CopyAndSetUpWorkroomPage(PageReference templateRef)
        {            
            PageReference newWorkroomRef = DataFactory.Instance.Copy(templateRef, CurrentPageLink);

            PageData newWorkroom = DataFactory.Instance.GetPage(newWorkroomRef).CreateWritableClone();

            newWorkroom.PageName = NewWorkroomName.Text;
            newWorkroom.URLSegment = UrlSegment.CreateUrlSegment(newWorkroom);
            DataFactory.Instance.Save(newWorkroom, SaveAction.Publish);

            return newWorkroom;
        }

        /// <summary>
        /// Creates the page folder of the workroom start page, if it isn't already created.
        /// </summary>
        /// <param name="workroomPage">The workroom start page.</param>
        private void CreateFolder(PageData workroomPage)
        {
            //Get the page folder id from the page.
            int folderId = (int)workroomPage["PageFolderID"];

            // Get the root dir from the VPP for Pages.
            string pageProviderRootVirtualPath = VirtualPathHandler.PageDirectoryRootVirtualPath;

            // Get the reference to this directory.
            IPageDirectory pageProviderRootDirectory = HostingEnvironment.VirtualPathProvider.GetDirectory(pageProviderRootVirtualPath) as IPageDirectory;
            if ((pageProviderRootDirectory != null) && (folderId > -1))
            {   
                //double check so the folder has not already been created 
                string newVirtualPath = VirtualPathUtilityEx.Combine(((UnifiedDirectory)pageProviderRootDirectory).VirtualPath, folderId.ToString());
                if (!HostingEnvironment.VirtualPathProvider.DirectoryExists(newVirtualPath))
                {
                    // Create the sub dir from page id.
                    UnifiedDirectory pageFolderDirectory = pageProviderRootDirectory.CreateSubdirectory(folderId.ToString(), workroomPage);
                }
            }
        }

        /// <summary>
        /// Clears any access rights on the workroom and adds the current user with full access.
        /// </summary>
        /// <param name="workroomPage">The workroom start page.</param>
        private void SetUpAccessRights(PageData workroomPage)
        {
            workroomPage.ACL.Clear();
            //TODO: Replace with security helper methods.
            workroomPage.ACL.Add(new AccessControlEntry(HttpContext.Current.User.Identity.Name, AccessLevel.FullAccess, SecurityEntityType.User));
            workroomPage.ACL.Save(SecuritySaveType.RecursiveReplace);
        }

        /// <summary>
        /// Validates that there isn't another workroom with the same name.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="args">The <see cref="System.Web.UI.WebControls.ServerValidateEventArgs"/> instance containing the event data.</param>
        protected void ValidateName(object source, ServerValidateEventArgs args)
        {
            args.IsValid = true;

            foreach (PageData page in DataFactory.Instance.GetChildren(CurrentPageLink))
            {
                if (String.Compare(args.Value, page.PageName, true) == 0)
                {
                    args.IsValid = false;
                    break;
                }
            }
        }

        /// <summary>
        /// Gets a list of all workrooms. Removes any that has where PageVisibleInMenu is false.
        /// </summary>
        /// <remarks>
        /// We need to manually remove pages with PageVisibleInMenu false since the PageList, which uses this property, doesn't filter that on it's own.
        /// </remarks>
        /// <value>A PageDataCollection containing all workrooms.</value>
        protected PageDataCollection Workrooms
        {
            get
            {
                if (_workrooms == null)
                {
                    _workrooms = DataFactory.Instance.GetChildren(CurrentPageLink);

                    FilterCompareTo filter = new FilterCompareTo("PageLink", CurrentPage["TemplateContainer"].ToString());
                    filter.Condition = CompareCondition.NotEqual;
                    filter.Filter(_workrooms);
                }
                return _workrooms;
            }
        }

        #region Button click event handlers

        /// <summary>
        /// Sets up a new workroom and redirects to that workroom when it's been set up.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CreateNewWorkroom_click(object sender, EventArgs e)
        {
            if (IsValid)
            {
                PageData newWorkroom = CopyAndSetUpWorkroomPage(PageReference.Parse(SelectTemplateDropDown.SelectedValue));

                CreateFolder(newWorkroom);

                SetUpAccessRights(newWorkroom);

                Response.Redirect(newWorkroom.LinkURL);
            }
        }

        /// <summary>
        /// Toggles the view between the workroom list panel and the create workroom panel.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void ToggleView_click(object sender, EventArgs e)
        {
            WorkroomListPanel.Visible = !WorkroomListPanel.Visible;
            NewWorkroomPanel.Visible = !NewWorkroomPanel.Visible;
        }

        #endregion
    }
}
