#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;

using EPiServer.Core;
using EPiServer.Filters;
using System.Text;

namespace EPiServer.Templates.Demo.Workroom.Units
{
    /// <summary>
    /// Lists X number of upcoming events.
    /// </summary>
    public partial class CalendarList : UserControlBase
    {
        // calendarEventPageTypeName is used to filter upcoming events.
        private static readonly string calendarEventPageTypeName = "[Demo] Workroom Calendar event";
        private static readonly string dateFormat = "dd MMM yyyy HH:mm";

        private PageDataCollection _upcomingEvents;
        private int _maxCount = 10;
        private PageReference _calendarContainer;
        private bool _listItemEven = true;

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            if (PageReference.IsNullOrEmpty(CalendarContainer))
            {
                this.Visible = false;
            }
            
            DataBind();
        }

        /// <summary>
        /// Gets a display string contain the start date and the end date of a calendar event.
        /// </summary>
        /// <param name="page">The PageData of the calendar event.</param>
        /// <returns>A string contain the start date and the end date of the calendar event.</returns>
        protected string GetDateString(PageData page)
        {
            if (page["EventStartDate"] == null || page["EventStopDate"] == null)
            {
                throw new EPiServerException("All calender event has to have the properties EventStartDate and EventStopDate defined");
            }

            StringBuilder sb = new StringBuilder();

            DateTime startDate = (DateTime)page["EventStartDate"];
            DateTime stopDate = (DateTime)page["EventStopDate"];

            if (startDate.Date == stopDate.Date)
            {
                sb.Append(startDate.ToString(dateFormat));
                sb.Append(" &#8722; ");
                sb.Append(stopDate.ToString("HH:mm"));
            }
            else
            {
                sb.Append(startDate.ToString(dateFormat));
                sb.Append(" &#8722; ");
                sb.Append(stopDate.ToString(dateFormat));
            }
            return sb.ToString();
        }

        #region Public properties

        /// <summary>
        /// Gets or sets a list with the upcoming calendar events.
        /// The list is sorted by the start date of the event, and will only contain the X number of closest upcoming events.
        /// </summary>
        /// <remarks>
        /// Can also be set to a custom PageDataCollection when calling the user control.
        /// </remarks>
        /// <value>A PageDataCollection with the upcoming events.</value>
        public PageDataCollection UpcomingEvents
        {
            get 
            {
                if (_upcomingEvents == null)
                {
                    if (PageReference.IsNullOrEmpty(CalendarContainer))
                    {
                        _upcomingEvents = new PageDataCollection();
                    }
                    else
                    {
                        _upcomingEvents = DataFactory.Instance.GetChildren(CalendarContainer);

                        _upcomingEvents = FilterForVisitor.Filter(_upcomingEvents);
                        new FilterCompareTo("PageTypeName", calendarEventPageTypeName).Filter(_upcomingEvents);
                        _upcomingEvents = ClearObsoleteEvents(_upcomingEvents);
                        new FilterPropertySort("EventStartDate", FilterSortDirection.Ascending).Filter(_upcomingEvents);
                        new FilterCount(MaxCount).Filter(_upcomingEvents);
                    }
                }
                return _upcomingEvents; 
            }
            set { _upcomingEvents = value; }
        }

        /// <summary>
        /// Gets or sets the number of events to list.
        /// </summary>
        /// <value>The number of events to list.</value>
        public int MaxCount
        {
            get { return _maxCount; }
            set { _maxCount = value; }
        }

        /// <summary>
        /// Gets or sets the calendar page reference.
        /// </summary>
        /// <value>The a PageReference to the calendar page.</value>
        public PageReference CalendarContainer
        {
            get { return _calendarContainer; }
            set { _calendarContainer = value; }
        }

        #endregion

        #region Private helper methods

        /// <summary>
        /// Gets a value indicating whether list items are in even (2,4,6...) or odd list positions.
        /// </summary>
        /// <value><c>true</c> if item is in even position otherwise, <c>false</c>.</value>
        protected bool ListItemEven
        {
            get
            {
                return _listItemEven = !_listItemEven;
            }
        }
        
        
        //Removes any events that has an EventStopDate that is in the past.
        private PageDataCollection ClearObsoleteEvents(PageDataCollection upcomingEvents)
        {
            PageDataCollection activeEvents = new PageDataCollection();

            foreach (PageData page in upcomingEvents)
            {
                if (((DateTime)page["EventStopDate"]) > DateTime.Now)
                {
                    activeEvents.Add(page);
                }
            }

            return activeEvents;
        }

        #endregion
    }
}