#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Hosting;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

using EPiServer.Core;
using EPiServer.Web.Hosting;
using System.IO;

namespace EPiServer.Templates.Demo.Workroom.Units
{
    /// <summary>
    /// Presents a list of most recent changed files
    /// </summary>
    [ToolboxData("<{0}:FileList runat=server></{0}:FileList>")]
    public partial class FileList : UserControlBase
    {
        // Used to set rendered file name length
        private static readonly int _allowedFileNameLength = 25;

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (String.IsNullOrEmpty(Heading))
            {
                FileListHeading.Visible = false;
            }

            FileListHeading.InnerText = Heading;

            if (!IsPostBack)
            {
                if (String.IsNullOrEmpty(RootFolder))
                {
                    ErrorMessage.Visible = true;
                    return;
                }
                
                List<UnifiedFile> lastModifiedFiles = FindLastModifiedFiles(RootFolder, MaxCount);
                if (lastModifiedFiles != null)
                {
                    FileListing.DataSource = lastModifiedFiles;
                    FileListing.DataBind();
                }
            }
        }

        #region Layout and Rendering Properties

        /// <summary>
        /// Gets or sets the CSS class for the File List.
        /// </summary>
        public string CssClass
        {
            get { return (string)(ViewState["CssClass"] ?? String.Empty); }
            set { ViewState["CssClass"] = value; }
        }
        
        /// <summary>
        /// Gets or sets the heading for the File List.
        /// </summary>
        public string Heading
        {
            get { return (string)(ViewState["Heading"] ?? String.Empty); }
            set { ViewState["Heading"] = value; }
        }

        /// <summary>
        /// Gets or sets the maximum number of files to show.
        /// </summary>
        /// <value>Number of files. Default is 5.</value>
        public int MaxCount
        {
            get { return (int)(ViewState["MaxCount"] ?? 5); }
            set { ViewState["MaxCount"] = value; }
        }
        
        /// <summary>
        /// Gets or sets the maximum number of files to show.
        /// </summary>
        /// <value>Number of files. Default is 5</value>
        public string RootFolder
        {
            get { return (string)(ViewState["RootFolder"] ?? String.Empty); }
            set { ViewState["RootFolder"] = value; }
        }
        #endregion


        /// <summary>
        /// Formats the length of the supplied file name.
        /// </summary>
        /// <param name="filename">The filename to format.</param>
        protected static string FormatFileNameLength(string filename)
        {
            if (filename.Length > _allowedFileNameLength)
            {
                return String.Format("{0}...", filename.Substring(0, _allowedFileNameLength));
            }
            else
            {
                return filename;
            }
        }

        /// <summary>
        /// Finds the last modified files.
        /// </summary>
        protected List<UnifiedFile> FindLastModifiedFiles(string rootFolder, int maxCount)
        {
            List<UnifiedFile> files = new List<UnifiedFile>();
            UnifiedDirectory directory = null;

            try
            {
                directory = HostingEnvironment.VirtualPathProvider.GetDirectory(rootFolder) as UnifiedDirectory;
            }
            catch (UnauthorizedAccessException) { }
            
            // If directory not found or access not granted for directory 
            // return empty list
            if (directory == null)
            {
                ShowErrorMessage(Translate("/workroom/filelist/errormessage"));
                return null;
            }
            
            UnifiedSearchQuery query = new UnifiedSearchQuery();
            query.ModifiedFrom = DateTime.MinValue;
            query.ModifiedTo = DateTime.Now;

            UnifiedSearchHitCollection hits = null;
            try
            {
                hits = directory.Search(query);
            }
            catch (IOException)
            {
                ShowErrorMessage(Translate("/workroom/filelist/errormissingindex"));
                return null;
            }

            for (int i = 0; i < hits.Count; i++)
            {
                try
                {
                    files.Add((UnifiedFile)HostingEnvironment.VirtualPathProvider.GetFile(hits[i].Path));
                }
                catch (UnauthorizedAccessException)
                {
                    // If user has limited access right we still want to show files with access granted.
                }
            }
            files.Sort(new FileModifiedComparer());

            if (files.Count > maxCount)
            {
                files.RemoveRange(maxCount - 1, files.Count - maxCount);
            }
            return files;
        }


        /// <summary>
        /// Sets an error message to the ErrorMessage literal control shows the control.
        /// </summary>
        /// <param name="message">The error message to show.</param>
        private void ShowErrorMessage(string message)
        {
            ErrorMessage.Text = message;
            ErrorMessage.Visible = true;
        }

    }

    /// <summary>
    /// Compares UnifiedFiles for when they are changed 
    /// </summary>
    public class FileModifiedComparer : IComparer<UnifiedFile>
    {
        #region IComparer<UnifiedFile> Members

        /// <summary>
        /// Compares two objects and returns a value indicating whether one is less than, equal to, or greater than the other.
        /// </summary>
        /// <param name="x">The first object to compare.</param>
        /// <param name="y">The second object to compare.</param>
        /// <returns>
        /// Value Condition Less than zero<paramref name="x"/> is less than <paramref name="y"/>.Zero<paramref name="x"/> equals <paramref name="y"/>.Greater than zero<paramref name="x"/> is greater than <paramref name="y"/>.
        /// </returns>
        public int Compare(UnifiedFile x, UnifiedFile y)
        {
            return y.Changed.CompareTo(x.Changed);
        }

        #endregion
    }
}