#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using EPiServer.Core;
using EPiServer.Templates.Demo.Workroom.Core;
using System.Collections.Generic;
using System.ComponentModel;

namespace EPiServer.Templates.Demo.Workroom.Units
{

    /// <summary>
    /// Implements user search and the possibility to modify the workroom membership level for listed users.
    /// </summary>
    public partial class MembershipAdministration : UserControlBase
    {
        // A collection of modified members. Will be stored in ViewState when changing pages in the user list.
        private List<MembershipEntry> _modifiedMembers = new List<MembershipEntry>();
        private PageData _workroomStartPage = null;

        /// <summary>
        /// Enumeration of available modes for this control
        /// </summary>
        private enum ViewMode 
        { 
            /// <summary>
            /// Only View the members with no editable fields
            /// </summary>
            ViewMembers,
            /// <summary>
            /// Edit the membership level of current members
            /// </summary>
            EditMembers,
            /// <summary>
            /// Search for and add new members
            /// </summary>
            AddMembers
        }

        private ViewMode ActiveMode
        {
            get { return (ViewMode)(ViewState["ActiveMode"] ?? ViewMode.ViewMembers); }
            set { ViewState["ActiveMode"] = value; }
        }

        /// <summary>
        /// Get the <see cref="EPiServer.Core.PageData"/> for the workroom start page.
        /// </summary>
        private PageData WorkroomStartPage
        {
            get
            {
                if (_workroomStartPage == null && Page is WorkroomPageBase)
                {
                    _workroomStartPage = ((WorkroomPageBase)Page).WorkroomStartPage;
                }
                return _workroomStartPage;
            }
        }

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            if (!IsPostBack)
            {
                SetViewMode(ViewMode.ViewMembers);
            }
        }

        /// <summary>
        /// Handles the DataBinding event of the MembershipLevel control in membership column of the the UserList gridview control.
        /// Used to populate the membership level drop down.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void MembershipLevel_DataBinding(object sender, EventArgs e)
        {
            Control container = (Control)sender;

            // We're inside a data row which is a data item container.
            IDataItemContainer dataContainer = container.BindingContainer as IDataItemContainer;

            if (dataContainer != null)
            {
                // Get the entry we're binding against.
                MembershipEntry userEntry = (MembershipEntry)dataContainer.DataItem;

                if ((ActiveMode == ViewMode.EditMembers && userEntry.Name != Context.User.Identity.Name) || (ActiveMode == ViewMode.AddMembers && userEntry.Membership == MembershipLevel.None))
                {
                    DropDownList membershipDropdown = (DropDownList)container.FindControl("MembershipDropdown");

                    foreach (MembershipLevel level in Enum.GetValues(typeof(MembershipLevel)))
                    {
                        membershipDropdown.Items.Add(new ListItem(Membership.TranslateMembershipLevel(level), userEntry.Name + ":" + level.ToString()));
                    }

                    // Check if the user has been modified but not saved.
                    MembershipEntry modifiedMembership;
                    int matchIndex = _modifiedMembers.IndexOf(userEntry);
                    if (matchIndex != -1)
                    {
                        modifiedMembership = _modifiedMembers[matchIndex];
                    }
                    else 
                    {
                        modifiedMembership = userEntry;
                    }

                    membershipDropdown.SelectedValue = userEntry.Name + ":" + modifiedMembership.Membership.ToString();
                    membershipDropdown.Visible = true;
                }
                else
                {
                    Literal membershipLiteral = (Literal)container.FindControl("MembershipLiteral");
                    membershipLiteral.Text = userEntry.MembershipLocalName;
                    membershipLiteral.Visible = true;
                }
            }
        }

        #region Control Event Handlers

        /// <summary>
        /// Updates the state of the control (visible buttons etc.) according to the new View Mode.
        /// </summary>
        /// <param name="viewMode">The new view mode.</param>
        private void SetViewMode(ViewMode viewMode)
        {
            bool isAdministrator = Membership.QueryDistinctMembershipLevel(WorkroomStartPage, MembershipLevel.Administer);
            if (!isAdministrator) 
            {
                viewMode = ViewMode.ViewMembers;
            }

            ActiveMode = viewMode;
            switch(viewMode)
            {
                case ViewMode.ViewMembers:
                    ToolbarButtons.Visible = isAdministrator;
                    PersistButtons.Visible = false;
                    SearchDialog.Visible = false;
                    MemberListHeading.Text = Translate("/workroom/membershipmanager/viewmembersheading");
                    break;
                case ViewMode.AddMembers:
                    ToolbarButtons.Visible = false;
                    PersistButtons.Visible = true;
                    SearchDialog.Visible = true;
                    MemberListHeading.Text = Translate("/workroom/membershipmanager/searchresultheading");
                    break;
                case ViewMode.EditMembers:
                    ToolbarButtons.Visible = false;
                    PersistButtons.Visible = true;
                    SearchDialog.Visible = false;
                    MemberListHeading.Text = Translate("/workroom/membershipmanager/managemembersheading");
                    break;
            }

            // Always clear the collection of modified members, 
            // this is only for persisting modifications between changes in paging.
            _modifiedMembers.Clear();
            UserList.PageIndex = 0;
            UserList.DataBind();
        }


        /// <summary>
        /// Handles the Click event of the EditMembers button.
        /// Shows all existing members with a permission drop down.
        /// </summary>
        protected void EditMembers_Click(object sender, EventArgs e)
        {
            SetViewMode(ViewMode.EditMembers);
        }

        /// <summary>
        /// Handles the Click event of the AddMembers button to enable edit of current members in the workroom.
        /// </summary>
        protected void AddMembers_Click(object sender, EventArgs e)
        {
            SetViewMode(ViewMode.AddMembers);
        }


        /// <summary>
        /// Handles the Click event of the CancelEdit to revert back to view mode of current members.
        /// </summary>
        protected void CancelEdit_Click(object sender, EventArgs e)
        {
            SetViewMode(ViewMode.ViewMembers);
        }

        /// <summary>
        /// Handles the Click event of the SaveMembership control to save the changes in membership.
        /// </summary>
        /// <param name="sender">The Save button.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void SaveMembership_Click(object sender, EventArgs e)
        {
            if (_modifiedMembers.Count > 0)
            {
                if (Membership.QueryDistinctMembershipLevel(WorkroomStartPage, MembershipLevel.Administer))
                {
                    Membership.ModifyPermission(WorkroomStartPage, _modifiedMembers);
                }
                _modifiedMembers.Clear();
            }
            SetViewMode(ViewMode.ViewMembers);
        }

        /// <summary>
        /// Handles the Click event of the SearchUsers control to databind the user list with search result.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void SearchUsers_Click(object sender, EventArgs e)
        {
            // Make sure to clear any membership modifications before showing the new search result.
            _modifiedMembers.Clear();
            UserList.PageIndex = 0;
            UserList.DataBind();
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the membershipDropdown control.
        /// When a membership level changes this is stored in the private collection _modifiedMembers, which is saved when the save button is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void MembershipDropdown_SelectedIndexChanged(object sender, EventArgs e)
        {
            DropDownList dropDown = (DropDownList)sender;
            string[] userValuePair = dropDown.SelectedValue.Split(':');
            MembershipEntry member = new MembershipEntry(userValuePair[0], (MembershipLevel)Enum.Parse(typeof(MembershipLevel), userValuePair[1]));
            if (_modifiedMembers.Contains(member))
            {
                _modifiedMembers.Remove(member);
            }
            _modifiedMembers.Add(member);
        }

        /// <summary>
        /// Executed when the Membership ObjectDataSource is selecting to decide whether to show 
        /// only existing members or a search result from all available users.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Web.UI.WebControls.ObjectDataSourceSelectingEventArgs"/> instance containing the event data.</param>
        protected void MemberObjectDataSource_Selecting(object sender, ObjectDataSourceSelectingEventArgs e)
        {
            e.InputParameters["onlyExistingMembers"] = (ActiveMode != ViewMode.AddMembers);
        }

        #endregion

        /// <summary>
        /// Saves any user control view-state changes that have occurred since the last page postback.
        /// </summary>
        /// <returns>
        /// Returns the user control's current view state. If there is no view state associated with the control, it returns null.
        /// </returns>
        protected override object SaveViewState()
        {
            ViewState["ModifiedMembers"] = _modifiedMembers;
            return base.SaveViewState();
        }

        /// <summary>
        /// Restores the view-state information from a previous user control request that was saved by the <see cref="M:System.Web.UI.UserControl.SaveViewState"/> method.
        /// </summary>
        /// <param name="savedState">An <see cref="T:System.Object"/> that represents the user control state to be restored.</param>
        protected override void LoadViewState(object savedState)
        {
            base.LoadViewState(savedState);
            _modifiedMembers = ViewState["ModifiedMembers"] as List<MembershipEntry>;
        }
    }
}