#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

using EPiServer.Core;
using EPiServer.Core.Html;
using EPiServer.Templates.Demo.Workroom.Core;
using EPiServer.Templates.Demo.Workroom.Pages;

namespace EPiServer.Templates.Demo.Workroom.Units
{
    public partial class NewsList : UserControlBase
    {
        /// <summary>
        /// Occurs when add button is clicked.
        /// </summary>
        public event EventHandler AddButtonClicked;

        private bool _listItemEven = true;
        private bool? _showPostBackLinks;
        private PageReference _selectedPageID;
        
        /// <summary>
        /// Sets the heading on the page.
        /// Loads the appropriate data to the MainIntroProp and MainBodyProp.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"></see> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (PageReference.IsNullOrEmpty(PageLink) && PageReference.IsNullOrEmpty((PageReference)CurrentPage[PageLinkProperty]) && Page as NewsListPage == null)
            {
                this.Visible = false;
            }
            
            if (!String.IsNullOrEmpty(Heading))
            {
                NewsListHeading.InnerText = Heading;
            }
            else
            {
                NewsListHeading.Visible = false;
            }
        }

        #region Public Properties

        /// <summary>
        /// Gets or sets the heading before the page list.
        /// </summary>
        public string Heading
        {
            get { return (string)(ViewState["Heading"] ?? String.Empty); }
            set { ViewState["Heading"] = value; }
        }

        /// <summary>
        /// Gets or sets the page property pointing out the parent page for news listing.
        /// </summary>
        /// <value>The name of the page property pointing out the parent page</value>
        public string PageLinkProperty
        {
            get { return NewsListing.PageLinkProperty; }
            set { NewsListing.PageLinkProperty = value; }
        }

        /// <summary>
        /// Gets or sets the page link for the parent for news listing.
        /// </summary>
        /// <value>PageReference PageLink</value>
        public PageReference PageLink
        {
            get { return NewsListing.PageLink; }
            set { NewsListing.PageLink = value; }
        }
       
        /// <summary>
        /// Gets or sets the maximum number of pages to show in listing.
        /// </summary>
        /// <value>Maximum number of pages to show</value>
        public int MaxCount
        {
            get { return NewsListing.MaxCount; }
            set { NewsListing.MaxCount = value; }
        }

        /// <summary>
        /// Gets or sets the length of the preview text.
        /// </summary>
        /// <value>PageReference PageLink. Default is 30</value>
        public int PreviewTextLength
        {
            get { return (int)(ViewState["PreviewTextLength"] ?? 30); }
            set { ViewState["PreviewTextLength"] = value; }
        }


        /// <summary>
        /// Enables paging of the list.
        /// Gets or sets the number of pages that goes into one paging item
        /// </summary>
        /// <value>Number of pages that goes into one paging item</value>
        public int PagesPerPagingItem
        {
            get 
            {
                return NewsListing.PagesPerPagingItem; 
            }
            set 
            {
                NewsListing.Paging = true;    
                NewsListing.PagesPerPagingItem = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating what type of links that should be used for the listing. Postback links, or hyper links.
        /// Postback links should only be used when the control is on pages of the type NewsListPage.
        /// </summary>
        /// <value><c>true</c> if PostBack links should be used; otherwise, <c>false</c>.</value>
        public bool ShowPostBackLinks
        {
            get 
            {
                if (_showPostBackLinks == null)
                {
                    if (Page as NewsListPage != null)
                    {
                        _showPostBackLinks = true;
                    }
                    else
                    {
                        _showPostBackLinks = false;
                    }
                }
                return _showPostBackLinks.Value; 
            }
            set { _showPostBackLinks = value; }
        }

        #endregion

        /// <summary>
        /// Gets a value indicating whether list items are in even (2,4,6...) or odd list positions.
        /// </summary>
        /// <value><c>true</c> if item is in even position otherwise, <c>false</c>.</value>
        protected bool ListItemEven
        {
            get 
            {
                return _listItemEven = !_listItemEven;
            }
        }


        /// <returns>Returns the preview text for the specified PageData</returns>
        /// <remarks>The preview text is primarily created from the MainIntro property if it exists, 
        /// otherwise parts of the MainBody property are being used. 
        /// If neither a MainIntro nor a MainBody property exists, the preview will not be shown. 
        /// The length of the preview text is defined in <code>PreviewTextLength</code></remarks>
        protected string GetPreviewText(PageData page)
        {
            if (PreviewTextLength <= 0)
            {
                return string.Empty;
            }

            string previewText = page["MainIntro"] as string;
            if (!String.IsNullOrEmpty(previewText))
            {
                return StripPreviewText(previewText, PreviewTextLength);
            }

            previewText = page["MainBody"] as string;
            if (String.IsNullOrEmpty(previewText))
            {
                return string.Empty;
            }

            return TextIndexer.StripHtml(previewText, PreviewTextLength);
        }

        /// <summary>
        /// Strips a text to a given length without splitting the last word.
        /// </summary>
        /// <param name="previewText">The string to shorten</param>
        /// <param name="maxLength">The max lenth of the string to return</param>
        /// <returns>A shortened version of the given string</returns>
        protected static string StripPreviewText(string previewText, int maxLength)
        {
            if (previewText.Length <= maxLength)
            {
                return previewText;
            }
            previewText = previewText.Substring(0, maxLength);
            // The maximum number of characters to cut from the end of the string.
            int maxCharCut = (previewText.Length > 15 ? 15 : previewText.Length - 1);
            int previousWord = previewText.LastIndexOfAny(new char[] { ' ', '.', ',', '!', '?' }, previewText.Length - 1, maxCharCut);
            if (previousWord <= 0)
            {
                previewText = previewText.Substring(0, previousWord);
            }
            return previewText + " ...";
        }

        /// <summary>
        /// Gets the navigate URL used for the links gerenrated in the PageList.
        /// </summary>
        /// <param name="selectedPage">The selected page.</param>
        /// <returns>A link to the current page with the id of the selected page as a query string parameter.</returns>
        protected string GetNavigateURL(PageData selectedPage)
        {   
            if (CurrentPage["NewsRoot"] == null)
            {
                return String.Empty;
            }

            StringBuilder link = new StringBuilder();

            link.Append(DataFactory.Instance.GetPage((PageReference)CurrentPage["NewsRoot"]).LinkURL);

            if (link.ToString().Contains("?"))
            {
                link.Append("&");
            }
            else
            {
                link.Append("?");
            }
            link.Append("selectednewspage=" + selectedPage.PageLink.ID);

            return link.ToString();
            
        }

        /// <summary>
        /// Handles the update of MainIntro and MainBody when an item in the news list is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Web.UI.WebControls.CommandEventArgs"/> instance containing the event data.</param>
        protected void LinkButton_Command(Object sender, CommandEventArgs e)
        {
            NewsListPage newsListPage = Page as NewsListPage;
            if (newsListPage != null)
            {
                PageReference pageRef = new PageReference((string)e.CommandArgument);
                SelectedPageID = pageRef;
                newsListPage.UpdateDisplayTexts(pageRef);
            }
        }

        /// <summary>
        /// Displays the create form on the NewsListPage that is hosting this control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void AddButton_Click(object sender, EventArgs e)
        {
            OnAddButtonClicked(new EventArgs());
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected virtual void OnAddButtonClicked(EventArgs e)
        {
            if (AddButtonClicked != null)
            {
                AddButtonClicked(this, e);
            }
        }

        /// <summary>
        /// Enables/disables the visibility of the add button. 
        /// </summary>
        public bool AddButtonVisible
        {
            get 
            {
                return AddButton.Visible;
            }
            set 
            {
                AddButton.Visible = value;
            }
        }

        /// <summary>
        /// Gets or sets the ID of the currently selected news item.
        /// </summary>
        /// <value>The selected page ID.</value>
        public PageReference SelectedPageID
        {
            get 
            {
                if (ViewState["SelectedPageID"] == null && _selectedPageID == null)
                {
                    ViewState["SelectedPageID"] = _selectedPageID = GetSelectedPageID();
                }
                else
                {
                    _selectedPageID = ViewState["SelectedPageID"] as PageReference;
                }
                return _selectedPageID;
            }
            set { ViewState["SelectedPageID"] = _selectedPageID = value; }
        }

        private PageReference GetSelectedPageID()
        {
            PageReference pageRef;

            //If there is a query string attribute called selectednewspage, use that.
            if (PageReference.TryParse(Request.QueryString["selectednewspage"], out pageRef))
            {
                return pageRef;
            }
            else
            {
                //Use the first item in the list as a default value.
                foreach (PageData page in NewsListing)
                {
                    return page.PageLink;
                }
            }

            return CurrentPage.PageLink;
        }
    }
}
